package asd.td3;

/**
 * Une classe pour l'exercice 5 de la feuille 3.
 * On teste rcursivement pour tout sites i et j si on peut se connecter
 * du site i au site j, en passant ventuellement par d'autres sites.
 *  2002 Marc Gatano
 */
public class Connexion {

    private boolean[][] c; // c[i][j] <==> il existe une connexion
                           // directe de i  j
    /**
     * Construit un objet qui represente les connexions directes possibles.
     * Paramtre : le tableau connexion tel que connexion[i][j] = true si on
     * peut se connecter directement de i a j.
     */
    public Connexion(boolean[][] connexion) {
        c = connexion;
    }

    /**
     * Teste si on peut se connecter du site i au site j, en passant
     * ventuellement par d'autres sites.
     * Complexit : O(3^n).
     */
    public boolean connecte(int i, int j) {
        return connecte(i,j,c.length);
    }

    /**
     * Teste si on peut se connecter de i  j en passant par un chemin
     * constitu de (zro, un, deux,..) sites appartenant  [0..k-1].
     * Complexit : O(3^k).
     */
    private boolean connecte(int i, int j, int k) {
        if ( k == 0 ) {
            return c[i][j];
        }
        return connecte(i,j,k-1) || connecte(i,k-1,k-1) && connecte(k-1,j,k-1);
    }
}
// Pour la complexit :
// La complexit de connecte(i,j,k) ne dpend que de k. Soit C(k) cette
// complexit. On a C(k) <= C(k-1) + C(k-1) + C(k-1) = 3*C(k-1), soit
// C(k) = O(3^k).
//
